@@ -113,8 +113,6 @@
 **
 */
 
-static const char rcsid[] = "$Id: acctproc.c,v 1.28 2010/04/23 12:20:19 gerlof Exp $";
-
 #define	_FILE_OFFSET_BITS	64
 
 #include <sys/types.h>
@@ -484,7 +482,17 @@ acctswon(void)
 	/*
 	** determine version of accounting-record
 	*/ 
-	(void) fstat(acctfd, &statbuf);
+	if (fstat(acctfd, &statbuf) == -1)
+	{
+		(void) acct(0);
+		(void) close(acctfd);
+		(void) unlink(ACCTDIR "/" ACCTFILE);
+		(void) rmdir(ACCTDIR);
+
+		acctfd = -1;
+		return 1;
+
+	}
 
 	if (statbuf.st_size == 0)	/* no acct record written yet */
 	{
@@ -657,7 +665,8 @@ acctprocnt(void)
 	/*
 	** determine the current size of the accounting file
 	*/
-	(void) fstat(acctfd, &statacc);
+	if (fstat(acctfd, &statacc) == -1)
+		return 0;
 
 	/*
  	** handle atopacctd-based process accounting on bases of
@@ -716,7 +725,8 @@ acctprocnt(void)
 		/*
 		** determine the size of newest shadow file
 		*/
-		(void) stat(shadowpath, &statacc);
+		if (stat(shadowpath, &statacc) == -1)
+			return numrecs;
 
 		numrecs += ((newseq - curshadowseq - 1) * maxshadowrec) +
 		           (statacc.st_size / acctrecsz);
@@ -783,7 +793,7 @@ acctrepos(unsigned int noverflow)
 ** read the process records from the process accounting file,
 ** that are written since the previous cycle
 */
-int
+unsigned long
 acctphotoproc(struct tstat *accproc, int nrprocs)
 {
 	register int 		nrexit;
@@ -802,7 +812,8 @@ acctphotoproc(struct tstat *accproc, int nrprocs)
 	** determine the size of the (current) account file
 	** and the current offset within that file
 	*/
-	(void) fstat(acctfd, &statacc);
+	if (fstat(acctfd, &statacc) == -1)
+		return 0;
 
 	/*
 	** check all exited processes in accounting file
@@ -812,7 +823,7 @@ acctphotoproc(struct tstat *accproc, int nrprocs)
 	{
 		/*
 		** in case of shadow accounting files, we might have to
-		** switch from the current accouting file to the next
+		** switch from the current accounting file to the next
 		*/
 		if (maxshadowrec && acctsize >= statacc.st_size)
 		{
@@ -822,7 +833,8 @@ acctphotoproc(struct tstat *accproc, int nrprocs)
 			** determine the size of the new (current) account file
 			** and initialize the current offset within that file
 			*/
-			(void) fstat(acctfd, &statacc);
+			if (fstat(acctfd, &statacc) == -1)
+				return 0;
 
 			acctsize  = 0;
 		}
