@@ -17,7 +17,7 @@
 ** Linux-port:  June 2000
 ** Modified: 	May 2001 - Ported to kernel 2.4
 ** --------------------------------------------------------------------------
-** Copyright (C) 2000-2012 Gerlof Langeveld
+** Copyright (C) 2000-2018 Gerlof Langeveld
 **
 ** This program is free software; you can redistribute it and/or modify it
 ** under the terms of the GNU General Public License as published by the
@@ -297,9 +297,9 @@ static const char rcsid[] = "$Id: atop.c,v 1.49 2010/10/23 14:01:00 gerlof Exp $
 #include "photosyst.h"
 #include "showgeneric.h"
 #include "parseable.h"
+#include "gpucom.h"
 
 #define	allflags  "ab:cde:fghijklmnopqrstuvwxyz1ABCDEFGHIJKL:MNOP:QRSTUVWXYZ"
-#define	PROCCHUNK	100	/* process-entries for future expansion  */
 #define	MAXFL		64      /* maximum number of command-line flags  */
 
 /*
@@ -325,6 +325,7 @@ char      	calcpss    = 0;  /* boolean: read/calculate process PSS  */
 
 unsigned short	hertz;
 unsigned int	pagesize;
+unsigned int	nrgpus;
 int 		osrel;
 int 		osvers;
 int 		ossub;
@@ -355,10 +356,12 @@ void do_linelength(char *, char *);
 void do_username(char *, char *);
 void do_procname(char *, char *);
 void do_maxcpu(char *, char *);
+void do_maxgpu(char *, char *);
 void do_maxdisk(char *, char *);
 void do_maxmdd(char *, char *);
 void do_maxlvm(char *, char *);
 void do_maxintf(char *, char *);
+void do_maxifb(char *, char *);
 void do_maxnfsm(char *, char *);
 void do_maxcont(char *, char *);
 void do_colinfo(char *, char *);
@@ -376,8 +379,10 @@ void do_owndskline(char *, char *);
 void do_ownnettransportline(char *, char *);
 void do_ownnetnetline(char *, char *);
 void do_ownnetinterfaceline(char *, char *);
+void do_owninfinibandline(char *, char *);
 void do_ownprocline(char *, char *);
 void do_cpucritperc(char *, char *);
+void do_gpucritperc(char *, char *);
 void do_memcritperc(char *, char *);
 void do_swpcritperc(char *, char *);
 void do_dskcritperc(char *, char *);
@@ -398,10 +403,12 @@ static struct {
 	{	"username",		do_username,		0, },
 	{	"procname",		do_procname,		0, },
 	{	"maxlinecpu",		do_maxcpu,		0, },
+	{	"maxlinegpu",		do_maxgpu,		0, },
 	{	"maxlinedisk",		do_maxdisk,		0, },
 	{	"maxlinemdd",		do_maxmdd,		0, },
 	{	"maxlinelvm",		do_maxlvm,		0, },
 	{	"maxlineintf",		do_maxintf,		0, },
+	{	"maxlineifb",		do_maxifb,		0, },
 	{	"maxlinenfsm",		do_maxnfsm,		0, },
 	{	"maxlinecont",		do_maxcont,		0, },
 	{	"colorinfo",		do_colinfo,		0, },
@@ -418,10 +425,12 @@ static struct {
 	{	"ownnettrline",		do_ownnettransportline,	0, },
 	{	"ownnetnetline",	do_ownnetnetline,	0, },
 	{	"ownnetifline",	        do_ownnetinterfaceline,	0, },
+	{	"ownifbline",	        do_owninfinibandline,	0, },
 	{	"ownprocline",		do_ownprocline,		0, },
 	{	"ownsysprcline",	do_ownsysprcline,	0, },
 	{	"owndskline",	        do_owndskline,		0, },
 	{	"cpucritperc",		do_cpucritperc,		0, },
+	{	"gpucritperc",		do_gpucritperc,		0, },
 	{	"memcritperc",		do_memcritperc,		0, },
 	{	"swpcritperc",		do_swpcritperc,		0, },
 	{	"dskcritperc",		do_dskcritperc,		0, },
@@ -573,7 +582,7 @@ main(int argc, char *argv[])
 		/*
 		** get optional interval-value and optional number of samples	
 		*/
-			if (optind < argc && optind < MAXFL)
+		if (optind < argc && optind < MAXFL)
 		{
 			if (!numeric(argv[optind]))
 				prusage(argv[0]);
@@ -648,14 +657,14 @@ main(int argc, char *argv[])
 
 	/*
 	** lock ATOP in memory to get reliable samples (also when
-	** memory is low);
+	** memory is low and swapping is going on);
 	** ignored if not running under superuser priviliges!
 	*/
 	rlim.rlim_cur	= RLIM_INFINITY;
 	rlim.rlim_max	= RLIM_INFINITY;
-	(void) setrlimit(RLIMIT_MEMLOCK, &rlim);
 
-	(void) mlockall(MCL_CURRENT|MCL_FUTURE);
+	if (setrlimit(RLIMIT_MEMLOCK, &rlim) == 0)
+		(void) mlockall(MCL_CURRENT|MCL_FUTURE);
 
 	/*
 	** increment CPU scheduling-priority to get reliable samples (also
@@ -680,9 +689,9 @@ main(int argc, char *argv[])
  	** open socket to the IP layer to issue getsockopt() calls later on
 	*/
 	netatop_ipopen();
-	
+
 	/*
-	** since priviliged activities are finished now, there is no
+	** since privileged activities are finished now, there is no
 	** need to keep running under root-priviliges, so switch
 	** effective user-id to real user-id
 	*/
@@ -721,17 +730,22 @@ engine(void)
 	** reserve space for task-level statistics
 	*/
 	static struct tstat	*curtpres;	/* current present list      */
-	static int		 curtlen;	/* size of present list      */
+	static unsigned long	 curtlen;	/* size of present list      */
 	struct tstat		*curpexit;	/* exited process list	     */
 
 	static struct devtstat	devtstat;	/* deviation info	     */
 
-	unsigned int		ntaskpres;	/* number of tasks present   */
-	unsigned int		nprocexit;	/* number of exited procs    */
-	unsigned int		nprocexitnet;	/* number of exited procs    */
+	unsigned long		ntaskpres;	/* number of tasks present   */
+	unsigned long		nprocexit;	/* number of exited procs    */
+	unsigned long		nprocexitnet;	/* number of exited procs    */
 						/* via netatopd daemon       */
 
-	unsigned int		noverflow;
+	unsigned long		noverflow;
+
+	int         		nrgpuproc=0,	/* number of GPU processes    */
+				gpupending=0;	/* boolean: request sent      */
+
+	struct gpupidstat	*gp = NULL;
 
 	/*
 	** initialization: allocate required memory dynamically
@@ -740,13 +754,9 @@ engine(void)
 	presstat = calloc(1, sizeof(struct sstat));
 	devsstat = calloc(1, sizeof(struct sstat));
 
-	curtlen  = countprocs() * 3 / 2;	/* add 50% for threads */
-	curtpres = calloc(curtlen, sizeof(struct tstat));
-
 	ptrverify(cursstat, "Malloc failed for current sysstats\n");
 	ptrverify(presstat, "Malloc failed for prev    sysstats\n");
 	ptrverify(devsstat, "Malloc failed for deviate sysstats\n");
-	ptrverify(curtpres, "Malloc failed for %d procstats\n", curtlen);
 
 	/*
 	** install the signal-handler for ALARM, USR1 and USR2 (triggers
@@ -779,6 +789,14 @@ engine(void)
 		timelimit = mktime(tp);
 	}
 
+	/*
+ 	** open socket to the atopgpud daemon for GPU statistics
+	*/
+        nrgpus = gpud_init();
+
+	if (nrgpus)
+		supportflags |= GPUSTAT;
+
 	/*
 	** MAIN-LOOP:
 	**    -	Wait for the requested number of seconds or for other trigger
@@ -820,6 +838,12 @@ engine(void)
 		pretime  = curtime;
 		curtime  = time(0);		/* seconds since 1-1-1970 */
 
+		/*
+		** send request for statistics to atopgpud 
+		*/
+		if (nrgpus)
+			gpupending = gpud_statrequest();
+
 		/*
 		** take a snapshot of the current system-level statistics 
 		** and calculate the deviations (i.e. calculate the activity
@@ -831,6 +855,45 @@ engine(void)
 
 		photosyst(cursstat);	/* obtain new counters      */
 
+		/*
+		** receive and parse response from atopgpud
+		*/
+		if (nrgpus && gpupending)
+		{
+			nrgpuproc = gpud_statresponse(nrgpus, cursstat->gpu.gpu, &gp);
+
+			gpupending = 0;
+
+			// connection lost or timeout on receive?
+			if (nrgpuproc == -1)
+			{
+				int ng;
+
+				// try to reconnect
+        			ng = gpud_init();
+
+				if (ng != nrgpus)	// no success
+					nrgpus = 0;
+
+				if (nrgpus)
+				{
+					// request for stats again
+					if (gpud_statrequest())
+					{
+						// receive stats response
+						nrgpuproc = gpud_statresponse(nrgpus,
+						     cursstat->gpu.gpu, &gp);
+
+						// persistent failure?
+						if (nrgpuproc == -1)
+							nrgpus = 0;
+					}
+				}
+			}
+
+			cursstat->gpu.nrgpus = nrgpus;
+		}
+
 		deviatsyst(cursstat, presstat, devsstat,
 				curtime-pretime > 0 ? curtime-pretime : 1);
 
@@ -840,23 +903,21 @@ engine(void)
 		** during the last sample)
 		**
 		** first register active tasks
-		**  --> atop malloc's a minimal amount of space which is
-		**      only extended when needed
 		*/
-		memset(curtpres, 0, curtlen * sizeof(struct tstat));
+		curtpres  = NULL;
 
-		while ( (ntaskpres = photoproc(curtpres, curtlen)) == curtlen)
+		do
 		{
-			curtlen += PROCCHUNK;
-
-			curtpres = realloc(curtpres,
+			curtlen   = counttasks();
+			curtpres  = realloc(curtpres,
 					curtlen * sizeof(struct tstat));
 
-			ptrverify(curtpres,
-			          "Realloc failed for %d tasks\n", curtlen);
+			ptrverify(curtpres, "Malloc failed for %d tstats\n",
+								curtlen);
 
 			memset(curtpres, 0, curtlen * sizeof(struct tstat));
 		}
+		while ( (ntaskpres = photoproc(curtpres, curtlen)) == curtlen);
 
 		/*
 		** register processes that exited during last sample;
@@ -893,7 +954,7 @@ engine(void)
 			curpexit = malloc(nprocexit * sizeof(struct tstat));
 
 			ptrverify(curpexit,
-			          "Malloc failed for %d exited processes\n",
+			          "Malloc failed for %lu exited processes\n",
 			          nprocexit);
 
 			memset(curpexit, 0, nprocexit * sizeof(struct tstat));
@@ -913,6 +974,14 @@ engine(void)
 			curpexit    = NULL;
 		}
 
+		/*
+ 		** merge GPU per-process stats with other per-process stats
+		*/
+		if (nrgpus && nrgpuproc)
+			gpumergeproc(curtpres, ntaskpres,
+		                     curpexit, nprocexit,
+		 	             gp,       nrgpuproc);
+
 		/*
 		** calculate deviations
 		*/
@@ -934,9 +1003,14 @@ engine(void)
 		if (nprocexit > 0)
 			free(curpexit);
 
+		free(curtpres);
+
 		if (nprocexitnet > 0)
 			netatop_exiterase();
 
+		if (gp)
+			 free(gp);
+
 		if (lastcmd == 'r')	/* reset requested ? */
 		{
 			sampcnt = -1;
@@ -945,7 +1019,6 @@ engine(void)
 
 			/* set current (will be 'previous') counters to 0 */
 			memset(cursstat, 0,           sizeof(struct sstat));
-			memset(curtpres, 0, curtlen * sizeof(struct tstat));
 
 			/* remove all tasks in database */
 			pdb_makeresidue();
