@@ -85,9 +85,11 @@
 void 	print_CPU();
 void 	print_cpu();
 void 	print_CPL();
+void 	print_GPU();
 void 	print_MEM();
 void 	print_SWP();
 void 	print_PAG();
+void 	print_PSI();
 void 	print_LVM();
 void 	print_MDD();
 void 	print_DSK();
@@ -95,16 +97,18 @@ void 	print_NFM();
 void 	print_NFC();
 void 	print_NFS();
 void 	print_NET();
+void 	print_IFB();
 
 void 	print_PRG();
 void 	print_PRC();
 void 	print_PRM();
 void 	print_PRD();
 void 	print_PRN();
+void 	print_PRE();
 
 /*
 ** table with possible labels and the corresponding
-** print-function for parseable output.
+** print-function for parseable output
 */
 struct labeldef {
 	char	*label;
@@ -116,9 +120,11 @@ static struct labeldef	labeldef[] = {
 	{ "CPU",	0,	print_CPU },
 	{ "cpu",	0,	print_cpu },
 	{ "CPL",	0,	print_CPL },
+	{ "GPU",	0,	print_GPU },
 	{ "MEM",	0,	print_MEM },
 	{ "SWP",	0,	print_SWP },
 	{ "PAG",	0,	print_PAG },
+	{ "PSI",	0,	print_PSI },
 	{ "LVM",	0,	print_LVM },
 	{ "MDD",	0,	print_MDD },
 	{ "DSK",	0,	print_DSK },
@@ -126,12 +132,14 @@ static struct labeldef	labeldef[] = {
 	{ "NFC",	0,	print_NFC },
 	{ "NFS",	0,	print_NFS },
 	{ "NET",	0,	print_NET },
+	{ "IFB",	0,	print_IFB },
 
 	{ "PRG",	0,	print_PRG },
 	{ "PRC",	0,	print_PRC },
 	{ "PRM",	0,	print_PRM },
 	{ "PRD",	0,	print_PRD },
 	{ "PRN",	0,	print_PRN },
+	{ "PRE",	0,	print_PRE },
 };
 
 static int	numlabels = sizeof labeldef/sizeof(struct labeldef);
@@ -311,7 +319,13 @@ print_CPU(char *hp, struct sstat *ss, struct tstat *ps, int nact)
         maxfreq = ss->cpu.cpu[0].freqcnt.maxfreq;
         calc_freqscale(maxfreq, cnt, ticks, &freq, &freqperc);
 
-	printf("%s %u %lld %lld %lld %lld %lld %lld %lld %lld %lld %lld %lld %d\n",
+	if (ss->cpu.all.instr == 1)
+	{
+        	ss->cpu.all.instr = 0;
+        	ss->cpu.all.cycle = 0;
+	}
+
+	printf("%s %u %lld %lld %lld %lld %lld %lld %lld %lld %lld %lld %lld %d %lld %lld\n",
 			hp,
 			hertz,
 	        	ss->cpu.nrcpu,
@@ -325,7 +339,9 @@ print_CPU(char *hp, struct sstat *ss, struct tstat *ps, int nact)
         		ss->cpu.all.steal,
         		ss->cpu.all.guest,
                         freq,
-                        freqperc
+                        freqperc,
+        		ss->cpu.all.instr,
+        		ss->cpu.all.cycle
                         );
 }
 
@@ -348,7 +364,7 @@ print_cpu(char *hp, struct sstat *ss, struct tstat *ps, int nact)
                 calc_freqscale(maxfreq, cnt, ticks, &freq, &freqperc);
 
 		printf("%s %u %d %lld %lld %lld "
-		       "%lld %lld %lld %lld %lld %lld %lld %d\n",
+		       "%lld %lld %lld %lld %lld %lld %lld %d %lld %lld\n",
 			hp, hertz, i,
 	        	ss->cpu.cpu[i].stime,
         		ss->cpu.cpu[i].utime,
@@ -360,7 +376,10 @@ print_cpu(char *hp, struct sstat *ss, struct tstat *ps, int nact)
         		ss->cpu.cpu[i].steal,
         		ss->cpu.cpu[i].guest,
                         freq,
-                        freqperc);
+                        freqperc,
+        		ss->cpu.cpu[i].instr,
+        		ss->cpu.cpu[i].cycle
+			);
 	}
 }
 
@@ -377,6 +396,28 @@ print_CPL(char *hp, struct sstat *ss, struct tstat *ps, int nact)
 	        	ss->cpu.devint);
 }
 
+void
+print_GPU(char *hp, struct sstat *ss, struct tstat *ps, int nact)
+{
+	int	i;
+
+	for (i=0; i < ss->gpu.nrgpus; i++)
+	{
+		printf("%s %d %s %s %d %d %lld %lld %lld %lld %lld %lld\n",
+			hp, i,
+	        	ss->gpu.gpu[i].busid,
+	        	ss->gpu.gpu[i].type,
+	        	ss->gpu.gpu[i].gpupercnow,
+	        	ss->gpu.gpu[i].mempercnow,
+	        	ss->gpu.gpu[i].memtotnow,
+	        	ss->gpu.gpu[i].memusenow,
+	        	ss->gpu.gpu[i].samples,
+	        	ss->gpu.gpu[i].gpuperccum,
+	        	ss->gpu.gpu[i].memperccum,
+	        	ss->gpu.gpu[i].memusecum);
+	}
+}
+
 void
 print_MEM(char *hp, struct sstat *ss, struct tstat *ps, int nact)
 {
@@ -415,7 +456,7 @@ print_SWP(char *hp, struct sstat *ss, struct tstat *ps, int nact)
 void
 print_PAG(char *hp, struct sstat *ss, struct tstat *ps, int nact)
 {
-	printf(	"%s %u %lld %lld %lld %lld %lld\n",
+	printf("%s %u %lld %lld %lld %lld %lld\n",
 			hp,
 			pagesize,
 			ss->mem.pgscans,
@@ -425,6 +466,24 @@ print_PAG(char *hp, struct sstat *ss, struct tstat *ps, int nact)
 			ss->mem.swouts);
 }
 
+void
+print_PSI(char *hp, struct sstat *ss, struct tstat *ps, int nact)
+{
+	printf("%s %c %.1f %.1f %.1f %llu %.1f %.1f %.1f %llu "
+	       "%.1f %.1f %.1f %llu %.1f %.1f %.1f %llu %.1f %.1f %.1f %llu\n",
+		hp, ss->psi.present ? 'y' : 'n',
+                ss->psi.cpusome.avg10, ss->psi.cpusome.avg60,
+                ss->psi.cpusome.avg300, ss->psi.cpusome.total,
+                ss->psi.memsome.avg10, ss->psi.memsome.avg60,
+                ss->psi.memsome.avg300, ss->psi.memsome.total,
+                ss->psi.memfull.avg10, ss->psi.memfull.avg60,
+                ss->psi.memfull.avg300, ss->psi.memfull.total,
+                ss->psi.iosome.avg10, ss->psi.iosome.avg60,
+                ss->psi.iosome.avg300, ss->psi.iosome.total,
+                ss->psi.iofull.avg10, ss->psi.iofull.avg60,
+                ss->psi.iofull.avg300, ss->psi.iofull.total);
+}
+
 void
 print_LVM(char *hp, struct sstat *ss, struct tstat *ps, int nact)
 {
@@ -490,7 +549,7 @@ print_NFM(char *hp, struct sstat *ss, struct tstat *ps, int nact)
 			hp,
 			ss->nfs.nfsmounts.nfsmnt[i].mountdev,
 			ss->nfs.nfsmounts.nfsmnt[i].bytestotread,
-			ss->nfs.nfsmounts.nfsmnt[i].bytestotread,
+			ss->nfs.nfsmounts.nfsmnt[i].bytestotwrite,
 			ss->nfs.nfsmounts.nfsmnt[i].bytesread,
 			ss->nfs.nfsmounts.nfsmnt[i].byteswrite,
 			ss->nfs.nfsmounts.nfsmnt[i].bytesdread,
@@ -519,8 +578,8 @@ print_NFS(char *hp, struct sstat *ss, struct tstat *ps, int nact)
 	        "%lld %lld %lld %lld %lld\n",
 			hp,
 			ss->nfs.server.rpccnt,
-			ss->nfs.client.rpcread,
-			ss->nfs.client.rpcwrite,
+			ss->nfs.server.rpcread,
+			ss->nfs.server.rpcwrite,
 			ss->nfs.server.nrbytes,
 			ss->nfs.server.nwbytes,
 			ss->nfs.server.rpcbadfmt,
@@ -572,16 +631,42 @@ print_NET(char *hp, struct sstat *ss, struct tstat *ps, int nact)
 	}
 }
 
+void
+print_IFB(char *hp, struct sstat *ss, struct tstat *ps, int nact)
+{
+	register int 	i;
+
+	for (i=0; i < ss->ifb.nrports; i++)
+	{
+		printf(	"%s %s %hd %hd %lld %lld %lld %lld %lld\n",
+			hp,
+			ss->ifb.ifb[i].ibname,
+			ss->ifb.ifb[i].portnr,
+			ss->ifb.ifb[i].lanes,
+			ss->ifb.ifb[i].rate,
+			ss->ifb.ifb[i].rcvb,
+			ss->ifb.ifb[i].sndb,
+			ss->ifb.ifb[i].rcvp,
+			ss->ifb.ifb[i].sndp);
+	}
+}
+
+
 /*
 ** print functions for process-level statistics
 */
 void
 print_PRG(char *hp, struct sstat *ss, struct tstat *ps, int nact)
 {
-	register int i;
+	register int i, exitcode;
 
 	for (i=0; i < nact; i++, ps++)
 	{
+		if (ps->gen.excode & 0xff)      // killed by signal?
+			exitcode = (ps->gen.excode & 0x7f) + 256;
+		else
+			exitcode = (ps->gen.excode >>   8) & 0xff;
+
 		printf("%s %d (%s) %c %d %d %d %d %d %ld (%s) %d %d %d %d "
  		       "%d %d %d %d %d %d %ld %c %d %d %s\n",
 			hp,
@@ -592,7 +677,7 @@ print_PRG(char *hp, struct sstat *ss, struct tstat *ps, int nact)
 			ps->gen.rgid,
 			ps->gen.tgid,
 			ps->gen.nthr,
-			ps->gen.excode,
+			exitcode,
 			ps->gen.btime,
 			ps->gen.cmdline,
 			ps->gen.ppid,
@@ -714,3 +799,26 @@ print_PRN(char *hp, struct sstat *ss, struct tstat *ps, int nact)
 				ps->gen.tgid,   ps->gen.isproc ? 'y':'n');
 	}
 }
+
+void
+print_PRE(char *hp, struct sstat *ss, struct tstat *ps, int nact)
+{
+	register int i;
+
+	for (i=0; i < nact; i++, ps++)
+	{
+		printf("%s %d (%s) %c %c %d %x %d %d %lld %lld %lld\n",
+				hp,
+				ps->gen.pid,
+				ps->gen.name,
+				ps->gen.state,
+				ps->gpu.state == '\0' ? 'N':ps->gpu.state,
+				ps->gpu.nrgpus,
+				ps->gpu.gpulist,
+				ps->gpu.gpubusy,
+				ps->gpu.membusy,
+				ps->gpu.memnow,
+				ps->gpu.memcum,
+				ps->gpu.sample);
+	}
+}
