@@ -169,7 +169,7 @@ static int	proccont(struct tstat *);
 static void	proccmd(struct tstat *);
 static void	procsmaps(struct tstat *);
 
-int
+unsigned long
 photoproc(struct tstat *tasklist, int maxtask)
 {
 	static int			firstcall = 1;
@@ -181,7 +181,7 @@ photoproc(struct tstat *tasklist, int maxtask)
 	DIR		*dirp;
 	struct dirent	*entp;
 	char		origdir[1024], dockstat=0;
-	int		tval=0;
+	unsigned long	tval=0;
 
 	/*
 	** one-time initialization stuff
@@ -225,10 +225,16 @@ photoproc(struct tstat *tasklist, int maxtask)
 	** read all subdirectory-names below the /proc directory
 	*/
 	if ( getcwd(origdir, sizeof origdir) == NULL)
+	{
+		perror("save current dir");
 		cleanstop(53);
+	}
 
 	if ( chdir("/proc") == -1)
-		cleanstop(53);
+	{
+		perror("change to /proc");
+		cleanstop(54);
+	}
 
 	dirp = opendir(".");
 
@@ -372,7 +378,10 @@ photoproc(struct tstat *tasklist, int maxtask)
 	closedir(dirp);
 
 	if ( chdir(origdir) == -1)
-		cleanstop(53);
+	{
+		perror(origdir);
+		cleanstop(55);
+	}
 
 	if (dockstat)
 		supportflags |= DOCKSTAT;
@@ -383,16 +392,41 @@ photoproc(struct tstat *tasklist, int maxtask)
 }
 
 /*
-** count number of processes currently running
+** count number of tasks in the system, i.e.
+** the number of processes plus the total number of threads
 */
-unsigned int
-countprocs(void)
+unsigned long
+counttasks(void)
 {
-	unsigned int	nr=0;
-	DIR		*dirp;
-	struct dirent	*entp;
-	char		origdir[1024];
+	unsigned long	nr=0;
+	char		linebuf[256];
+	FILE		*fp;
+	DIR             *dirp;
+	struct dirent   *entp;
+	char            origdir[1024];
 
+	/*
+	** determine total number of threads 
+	*/
+	if ( (fp = fopen("/proc/loadavg", "r")) != NULL)
+	{
+		if ( fgets(linebuf, sizeof(linebuf), fp) != NULL)
+		{
+			if ( sscanf(linebuf, "%*f %*f %*f %*d/%lu", &nr) < 1)
+				cleanstop(53);
+		}
+		else
+			cleanstop(53);
+
+		fclose(fp);
+	}
+	else
+		cleanstop(53);
+
+
+	/*
+	** add total number of processes 
+	*/
 	if ( getcwd(origdir, sizeof origdir) == NULL)
 		cleanstop(53);
 
@@ -404,7 +438,7 @@ countprocs(void)
 	while ( (entp = readdir(dirp)) )
 	{
 		/*
-		** count subdirectory-names starting with a digit
+		** count subdirectory names under /proc starting with a digit
 		*/
 		if (isdigit(entp->d_name[0]))
 			nr++;
@@ -468,7 +502,7 @@ procstat(struct tstat *curtask, unsigned long long bootepoch, char isproc)
 	/*
   	** fetch other values
   	*/
-	curtask->gen.isproc = isproc;
+	curtask->gen.isproc  = isproc;
 	curtask->cpu.rtprio  = 0;
 	curtask->cpu.policy  = 0;
 	curtask->gen.excode  = 0;
