@@ -164,6 +164,16 @@ static const char rcsid[] = "$Id: photosyst.c,v 1.38 2010/11/19 07:40:40 gerlof
 #include <signal.h>
 #include <string.h>
 #include <dirent.h>
+#include <sys/ioctl.h>
+#include <sys/sysmacros.h>
+
+#define SCALINGMAXCPU	8	// threshold for scaling info per CPU
+
+
+#ifndef	NOPERFEVENT
+#include <linux/perf_event.h>
+#include <asm/unistd.h>
+#endif
 
 #include <sys/socket.h>
 #include <netinet/in.h>
@@ -184,6 +194,12 @@ static const char rcsid[] = "$Id: photosyst.c,v 1.38 2010/11/19 07:40:40 gerlof
 #define	MDDTYPE	2
 #define	LVMTYPE	3
 
+#ifndef	NOPERFEVENT
+static void	getperfevents(struct cpustat *);
+#endif
+
+static int	get_infiniband(struct ifbstat *);
+
 static int	isdisk(unsigned int, unsigned int,
 			char *, struct perdsk *, int);
 
@@ -267,12 +283,14 @@ static int	v6tab_entries = sizeof(v6tab)/sizeof(struct v6tab);
 void
 photosyst(struct sstat *si)
 {
+	static char	part_stats = 1; /* per-partition statistics ? */
+	static char	ib_stats = 1; 	/* InfiniBand statistics ? */
+
 	register int	i, nr;
 	count_t		cnts[MAXCNT];
 	float		lavg1, lavg5, lavg15;
 	FILE 		*fp;
 	char		linebuf[1024], nam[64], origdir[1024];
-	static char	part_stats = 1; /* per-partition statistics ? */
 	unsigned int	major, minor;
 	struct shm_info	shminfo;
 #if	HTTPSTATS
@@ -282,10 +300,17 @@ photosyst(struct sstat *si)
 	memset(si, 0, sizeof(struct sstat));
 
 	if ( getcwd(origdir, sizeof origdir) == NULL)
+	{
+		perror("save current dir");
 		cleanstop(53);
+	}
 
 	if ( chdir("/proc") == -1)
-		cleanstop(53);
+	{
+		perror("change to /proc");
+		cleanstop(54);
+	}
+
 
 	/*
 	** gather various general statistics from the file /proc/stat and
@@ -426,11 +451,13 @@ photosyst(struct sstat *si)
         **
 	** store them in binary form
 	*/
-        static char fn[80];
+        static char fn[256];
         int didone=0;
 
-        for (i = 0; i < si->cpu.nrcpu; ++i)
-        {
+	if (si->cpu.nrcpu <= SCALINGMAXCPU)
+	{
+            for (i = 0; i < si->cpu.nrcpu; ++i)
+            {
                 long long f=0;
 
                 sprintf(fn,
@@ -504,7 +531,8 @@ photosyst(struct sstat *si)
                                 si->cpu.cpu[i].freqcnt.ticks 	= 0;
                         }
                 }
-        } // for all CPUs
+            } // for all CPUs
+	}
 
         if (!didone)     // did not get processor freq statistics.
                          // use /proc/cpuinfo
@@ -1266,6 +1294,101 @@ photosyst(struct sstat *si)
 	if (! droprootprivs())
 		cleanstop(42);
 
+	/*
+	** pressure statistics in /proc/pressure (>= 4.20)
+	**
+	** cpu:      some avg10=0.00 avg60=1.37 avg300=3.73 total=30995960
+	** io:       some avg10=0.00 avg60=8.83 avg300=22.86 total=141658568
+	** io:       full avg10=0.00 avg60=8.33 avg300=21.56 total=133129045
+	** memory:   some avg10=0.00 avg60=0.74 avg300=1.67 total=10663184
+	** memory:   full avg10=0.00 avg60=0.45 avg300=0.94 total=6461782
+	**
+	** verify if pressure stats supported by this system
+	*/
+	if ( chdir("pressure") == 0)
+ 	{
+		struct psi 	psitemp;
+		char 		psitype;
+		char 		psiformat[] =
+				"%c%*s avg10=%f avg60=%f avg300=%f total=%llu";
+
+		si->psi.present = 1;
+
+		if ( (fp = fopen("cpu", "r")) != NULL)
+		{
+			if ( fgets(linebuf, sizeof(linebuf), fp) != NULL)
+			{
+				nr = sscanf(linebuf, psiformat,
+			            	&psitype,
+					&psitemp.avg10, &psitemp.avg60,
+					&psitemp.avg300, &psitemp.total);
+
+				if (nr == 5)	// complete line ?
+					memmove(&(si->psi.cpusome), &psitemp,
+								sizeof psitemp);
+			}
+			fclose(fp);
+		}
+
+		if ( (fp = fopen("memory", "r")) != NULL)
+		{
+			while ( fgets(linebuf, sizeof(linebuf), fp) != NULL)
+			{
+				nr = sscanf(linebuf, psiformat,
+			            	&psitype,
+					&psitemp.avg10, &psitemp.avg60,
+					&psitemp.avg300, &psitemp.total);
+
+				if (nr == 5)
+				{
+					if (psitype == 's')
+						memmove(&(si->psi.memsome),
+							&psitemp,
+							sizeof psitemp);
+					else
+						memmove(&(si->psi.memfull),
+							&psitemp,
+							sizeof psitemp);
+				}
+			}
+			fclose(fp);
+		}
+
+		if ( (fp = fopen("io", "r")) != NULL)
+		{
+			while ( fgets(linebuf, sizeof(linebuf), fp) != NULL)
+			{
+				nr = sscanf(linebuf, psiformat,
+			            	&psitype,
+					&psitemp.avg10, &psitemp.avg60,
+					&psitemp.avg300, &psitemp.total);
+
+				if (nr == 5)
+				{
+					if (psitype == 's')
+						memmove(&(si->psi.iosome),
+							&psitemp,
+							sizeof psitemp);
+					else
+						memmove(&(si->psi.iofull),
+							&psitemp,
+							sizeof psitemp);
+				}
+			}
+			fclose(fp);
+		}
+
+		if ( chdir("..") == -1)
+		{
+			perror("return to /proc");
+			cleanstop(54);
+		}
+	}
+	else
+	{
+		si->psi.present = 0;
+	}
+
 	/*
 	** Container statistics (if any)
 	*/
@@ -1361,11 +1484,28 @@ photosyst(struct sstat *si)
 		}
 	}
 
+	/*
+ 	** verify presence of InfiniBand controllers
+	** warning: possibly switches to other directory
+	*/
+	if (ib_stats)
+		ib_stats = get_infiniband(&(si->ifb));
+
 	/*
  	** return to original directory
 	*/
 	if ( chdir(origdir) == -1)
-		cleanstop(53);
+	{
+		perror(origdir);
+		cleanstop(55);
+	}
+
+#ifndef	NOPERFEVENT
+	/*
+	** get low-level CPU event counters
+	*/
+        getperfevents(&(si->cpu));
+#endif
 
 	/*
 	** fetch application-specific counters
@@ -1528,6 +1668,8 @@ static struct {
 	{ "^dm-[0-9][0-9]*$",			{0},  lvmmapname,  LVMTYPE, },
 	{ "^md[0-9][0-9]*$",			{0},  nullmodname, MDDTYPE, },
 	{ "^vd[a-z][a-z]*$",                    {0},  nullmodname, DSKTYPE, },
+	{ "^nvme[0-9][0-9]*n[0-9][0-9]*$",	{0},  nullmodname, DSKTYPE, },
+	{ "^nbd[0-9][0-9]*$",			{0},  nullmodname, DSKTYPE, },
 	{ "^hd[a-z]$",				{0},  nullmodname, DSKTYPE, },
 	{ "^rd/c[0-9][0-9]*d[0-9][0-9]*$",	{0},  nullmodname, DSKTYPE, },
 	{ "^cciss/c[0-9][0-9]*d[0-9][0-9]*$",	{0},  nullmodname, DSKTYPE, },
@@ -1637,6 +1779,387 @@ getbootlinux(long hertz)
 	return bootjiffies;
 }
 
+
+/*
+** get stats of all InfiniBand ports below
+**    /sys/class/infiniband/<controller>/ports/<port#>/....
+*/
+static struct ibcachent {
+	char		*ibha;		// InfiniBand Host Adaptor
+	unsigned short	port;		// port number
+	unsigned short	lanes;		// number of lanes
+	count_t 	rate;		// transfer rate in bytes/sec
+	char 		*pathrcvb;	// path name for received bytes
+	char 		*pathsndb;	// path name for transmitted bytes
+	char 		*pathrcvp;	// path name for received packets
+	char 		*pathsndp;	// path name for transmitted packets
+} ibcache[MAXIBPORT];
+
+static int	nib;			// number of IB ports in cache
+
+static void	ibprep(struct ibcachent *);
+static int	ibstat(struct ibcachent *, struct perifb *);
+
+static int
+get_infiniband(struct ifbstat *si)
+{
+	static int	firstcall = 1;
+	int		i;
+
+	// verify if InfiniBand used in this system
+	if ( chdir("/sys/class/infiniband") == -1)
+		return 0;	// no path, no IB, so don't try again
+
+	if (firstcall)
+	{
+		char		path[128], *p;
+		struct stat	statbuf;
+		struct dirent	*contdent, *portdent;
+		DIR		*contp, *portp;
+
+		firstcall = 0;
+
+		/*
+		** once setup a cache with all info that is needed
+		** to gather the necessary stats with every subsequent
+		** call, including  path names, etcetera.
+		*/
+		if ( (contp = opendir(".")) )
+		{
+			/*
+ 			** read every directory-entry and search for
+			** subdirectories (i.e. controllers)
+			*/
+			while ( (contdent = readdir(contp)) )
+			{
+				// skip . and ..
+				if (contdent->d_name[0] == '.')
+					continue;
+
+				if ( stat(contdent->d_name, &statbuf) == -1 )
+					continue;
+	
+				if ( ! S_ISDIR(statbuf.st_mode) )
+					continue;
+
+				// controller found
+				// store controller name for cache
+				//
+				p = malloc( strlen(contdent->d_name)+1 );
+				strcpy(p, contdent->d_name);
+
+				if (strlen(contdent->d_name) > MAXIBNAME-1)
+					p[MAXIBNAME-1] = '\0';
+
+				// discover all ports
+				//
+				snprintf(path, sizeof path, "%s/ports",
+							contdent->d_name);
+
+				if ( (portp = opendir(path)) )
+				{
+					/*
+ 					** read every directory-entry and
+					** search for subdirectories (i.e.
+					** port numbers)
+					*/
+					while ( (portdent = readdir(portp)) )
+					{
+						int port;
+
+						// skip . and ..
+						if (portdent->d_name[0] == '.')
+							continue;
+
+						port = atoi(portdent->d_name);
+
+						if (!port)
+							continue;
+
+						// valid port
+						// fill cache info
+						//
+						ibcache[nib].port = port;
+						ibcache[nib].ibha = p;
+
+						ibprep(&ibcache[nib]);
+
+						if (++nib >= MAXIBPORT)
+							break;
+					}
+
+					closedir(portp);
+
+					if (nib >= MAXIBPORT)
+						break;
+				}
+			}
+
+			closedir(contp);
+		}
+	}
+
+	/*
+	** get static and variable metrics
+	*/
+	for (i=0; i < nib; i++)
+	{
+		// static metrics from cache
+		strcpy(si->ifb[i].ibname, ibcache[i].ibha);
+
+		si->ifb[i].portnr = ibcache[i].port;
+		si->ifb[i].lanes  = ibcache[i].lanes;
+		si->ifb[i].rate   = ibcache[i].rate;
+
+		// variable metrics from sysfs
+		ibstat(&(ibcache[i]), &(si->ifb[i]));
+	}	
+
+	si->nrports = nib;
+	return 1;
+}
+
+/*
+** determine rate and number of lanes
+** from <contr>/ports/<port>/rate --> e.g. "100 Gb/sec (4X EDR)"
+** and assemble path names to be used for counters later on
+*/
+static void
+ibprep(struct ibcachent *ibc)
+{
+	FILE	*fp;
+	char	path[128], linebuf[64], speedunit;
+
+	// determine port rate and number of lanes
+	snprintf(path, sizeof path, "%s/ports/%d/rate", ibc->ibha, ibc->port); 
+
+	if ( (fp = fopen(path, "r")) )
+	{
+		if ( fgets(linebuf, sizeof(linebuf), fp) != NULL)
+		{
+			(void) sscanf(linebuf, "%lld %c%*s (%hdX",
+				&(ibc->rate), &speedunit, &(ibc->lanes));
+		}
+
+		fclose(fp);
+	}
+
+	// calculate megabits/second
+	switch (speedunit)
+	{
+	   case 'M':
+	   case 'm':
+		break;
+	   case 'G':
+	   case 'g':
+		ibc->rate *= 1000;
+		break;
+	   case 'T':
+	   case 't':
+		ibc->rate *= 1000000;
+		break;
+	}
+
+	// build all pathnames to obtain the counters
+	// of this port later on
+	snprintf(path, sizeof path, "%s/ports/%d/counters/port_rcv_data",
+						    ibc->ibha, ibc->port);
+	ibc->pathrcvb = malloc( strlen(path)+1 );
+	strcpy(ibc->pathrcvb, path);
+
+	snprintf(path, sizeof path, "%s/ports/%d/counters/port_xmit_data",
+						    ibc->ibha, ibc->port);
+	ibc->pathsndb = malloc( strlen(path)+1 );
+	strcpy(ibc->pathsndb, path);
+
+	snprintf(path, sizeof path, "%s/ports/%d/counters/port_rcv_packets",
+						    ibc->ibha, ibc->port);
+	ibc->pathrcvp = malloc( strlen(path)+1 );
+	strcpy(ibc->pathrcvp, path);
+
+	snprintf(path, sizeof path, "%s/ports/%d/counters/port_xmit_packets",
+						    ibc->ibha, ibc->port);
+	ibc->pathsndp = malloc( strlen(path)+1 );
+	strcpy(ibc->pathsndp, path);
+}
+
+/*
+** read necessary variable counters for this IB port
+*/
+static int
+ibstat(struct ibcachent *ibc, struct perifb *ifb)
+{
+	FILE	*fp;
+	char	linebuf[64];
+
+	if ( (fp = fopen(ibc->pathrcvb, "r")) )
+	{
+		if ( fgets(linebuf, sizeof(linebuf), fp) != NULL)
+		{
+			if (sscanf(linebuf, "%lld", &(ifb->rcvb)) == 0)
+				ifb->rcvb = 0;
+		}
+		else
+		{
+			ifb->rcvb = 0;
+		}
+		fclose(fp);
+	}
+
+	if ( (fp = fopen(ibc->pathsndb, "r")) )
+	{
+		if ( fgets(linebuf, sizeof(linebuf), fp) != NULL)
+		{
+			if (sscanf(linebuf, "%lld", &(ifb->sndb)) == 0)
+				ifb->sndb = 0;
+		}
+		else
+		{
+			ifb->sndb = 0;
+		}
+		fclose(fp);
+	}
+
+	if ( (fp = fopen(ibc->pathrcvp, "r")) )
+	{
+		if ( fgets(linebuf, sizeof(linebuf), fp) != NULL)
+		{
+			if (sscanf(linebuf, "%lld", &(ifb->rcvp)) == 0)
+				ifb->rcvp = 0;
+		}
+		else
+		{
+			ifb->rcvp = 0;
+		}
+		fclose(fp);
+	}
+
+	if ( (fp = fopen(ibc->pathsndp, "r")) )
+	{
+		if ( fgets(linebuf, sizeof(linebuf), fp) != NULL)
+		{
+			if (sscanf(linebuf, "%lld", &(ifb->sndp)) == 0)
+				ifb->sndp = 0;
+		}
+		else
+		{
+			ifb->sndp = 0;
+		}
+		fclose(fp);
+	}
+
+	return 1;
+}
+
+/*
+** retrieve low-level CPU events:
+** 	instructions and cycles per CPU
+*/
+#ifndef	NOPERFEVENT
+long
+perf_event_open(struct perf_event_attr *hwevent, pid_t pid,
+                int cpu, int groupfd, unsigned long flags)
+{
+	return syscall(__NR_perf_event_open, hwevent, pid, cpu, groupfd, flags);
+}
+
+static void
+getperfevents(struct cpustat *cs)
+{
+	static int	firstcall = 1, cpualloced, *fdi, *fdc;
+	int		i;
+
+	/*
+ 	** once initialize perf event counter retrieval
+	*/
+	if (firstcall)
+	{
+		struct perf_event_attr  pea;
+		int			success = 0;
+
+		firstcall = 0;
+
+		/*
+		** allocate space for per-cpu file descriptors
+		*/
+		cpualloced = cs->nrcpu;
+		fdi        = malloc(sizeof(int) * cpualloced);
+		fdc        = malloc(sizeof(int) * cpualloced);
+
+		/*
+		** fill perf_event_attr struct with appropriate values
+		*/
+		memset(&pea, 0, sizeof(struct perf_event_attr));
+
+		pea.type    = PERF_TYPE_HARDWARE;
+		pea.size    = sizeof(struct perf_event_attr);
+		pea.inherit = 1;
+		pea.pinned  = 1;
+
+	 	regainrootprivs();
+
+		for (i=0; i < cpualloced; i++)
+		{
+			pea.config = PERF_COUNT_HW_INSTRUCTIONS;
+
+			if ( (*(fdi+i) = perf_event_open(&pea, -1, i, -1,
+ 						PERF_FLAG_FD_CLOEXEC)) >= 0)
+				success++;
+
+                        pea.config = PERF_COUNT_HW_CPU_CYCLES;
+
+			if ( (*(fdc+i) = perf_event_open(&pea, -1, i, -1,
+						PERF_FLAG_FD_CLOEXEC)) >= 0)
+				success++;
+		}
+
+		if (! droprootprivs())
+			cleanstop(42);
+
+		/*
+		** all failed (probably no kernel support)?
+		*/
+		if (success == 0)	
+		{
+			free(fdi);
+			free(fdc);
+			cpualloced = 0;
+		}
+		else
+		{
+			cs->all.instr = 1;
+			cs->all.cycle = 1;
+		}
+
+		return;		// initialization finished for first sample
+        }
+
+	/*
+	** every sample: check if counters available anyhow
+	*/
+        if (!cpualloced)
+                return;
+
+	/*
+   	** retrieve counters per CPU and in total
+	*/
+	cs->all.instr = 0;
+	cs->all.cycle = 0;
+
+        for (i=0; i < cpualloced; i++)
+        {
+		if (*(fdi+i) != -1)
+		{
+                	read(*(fdi+i), &(cs->cpu[i].instr), sizeof(count_t));
+                        cs->all.instr += cs->cpu[i].instr;
+
+                	read(*(fdc+i), &(cs->cpu[i].cycle), sizeof(count_t));
+                        cs->all.cycle += cs->cpu[i].cycle;
+		}
+        }
+}
+#endif
+
 #if	HTTPSTATS
 /*
 ** retrieve statistics from local HTTP daemons
