@@ -21,6 +21,9 @@
 ** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 ** See the GNU General Public License for more details.
 */
+#ifndef __PHOTOSYST__
+#define __PHOTOSYST__
+
 #include "netstats.h"
 
 #define	MAXCPU		2048
@@ -30,11 +33,15 @@
 #define	MAXINTF		128
 #define	MAXCONTAINER	128
 #define	MAXNFSMOUNT	64
+#define	MAXIBPORT	32
+#define	MAXGPU		32
+#define	MAXGPUBUS	12
+#define	MAXGPUTYPE	12
 
 #define	MAXDKNAM	32
+#define	MAXIBNAME	12
 
 /************************************************************************/
-
 struct	memstat {
 	count_t	physmem;	// number of physical pages
 	count_t	freemem;	// number of free     pages
@@ -66,7 +73,6 @@ struct	memstat {
 	count_t	hugepagesz;	// huge page size (bytes)
 
 	count_t	vmwballoon;	// vmware claimed balloon pages
-
 	count_t	cfuture[8];	// reserved for future use
 };
 
@@ -105,7 +111,9 @@ struct percpu {
 	count_t		steal;	/* steal   time in clock ticks		*/
 	count_t		guest;	/* guest   time in clock ticks		*/
         struct freqcnt	freqcnt;/* frequency scaling info  		*/
-	count_t		cfuture[4];	/* reserved for future use	*/
+	count_t		instr;	/* CPU instructions 			*/
+	count_t		cycle;	/* CPU cycles 				*/
+	count_t		cfuture[2];	/* reserved for future use	*/
 };
 
 struct	cpustat {
@@ -241,6 +249,24 @@ struct nfsstat {
 	} nfsmounts;
 };
 
+/************************************************************************/
+struct	psi {
+	float	avg10;		// average pressure last 10 seconds
+	float	avg60;		// average pressure last 60 seconds
+	float	avg300;		// average pressure last 300 seconds
+	count_t	total;		// total number of milliseconds
+};
+
+struct	pressure {
+	char	   present;	/* pressure stats supported?	*/
+	char       future[3];
+	struct psi cpusome;	/* pressure stall info 'some'   */
+	struct psi memsome;	/* pressure stall info 'some'   */
+	struct psi memfull;	/* pressure stall info 'full'   */
+	struct psi iosome;	/* pressure stall info 'some'   */
+	struct psi iofull;	/* pressure stall info 'full'   */
+};
+
 /************************************************************************/
 
 struct  percontainer {
@@ -278,6 +304,49 @@ struct wwwstat {
 int	getwwwstat(unsigned short, struct wwwstat *);
 #endif
 /************************************************************************/
+struct pergpu {
+	char	taskstats;		// GPU task statistics supported?
+	unsigned char   nrprocs;	// number of processes using GPU
+	char	type[MAXGPUTYPE+1];	// GPU type
+	char	busid[MAXGPUBUS+1];	// GPU bus identification
+	int	gpunr;			// GPU number
+	int	gpupercnow;		// processor percentage last second
+					// -1 if not supported
+	int	mempercnow;		// memory    percentage last second
+					// -1 if not supported
+	count_t	memtotnow;		// total memory in KiB
+	count_t	memusenow;		// used  memory in KiB
+	count_t	samples;		// number of samples
+	count_t	gpuperccum;		// cumulative processor busy percentage
+					// -1 if not supported
+	count_t	memperccum;		// cumulative memory percentage 
+					// -1 if not supported
+	count_t	memusecum;		// cumulative used memory in KiB
+};
+
+struct gpustat {
+	int		nrgpus;		// total number of GPUs
+	struct pergpu   gpu[MAXGPU];
+};
+
+/************************************************************************/
+struct perifb {
+	char	ibname[MAXIBNAME];	// InfiniBand controller
+	short	portnr;			// InfiniBand controller port
+
+	short	lanes;			// number of lanes (traffic factor)
+	count_t	rate;			// transfer rate in megabits/sec
+	count_t	rcvb;   	    	// bytes received
+	count_t	sndb;       		// bytes transmitted
+	count_t	rcvp;   	    	// packets received
+	count_t	sndp;       		// packets transmitted
+};
+
+struct ifbstat {
+	int		nrports;	// total number of IB ports
+	struct perifb   ifb[MAXIBPORT];
+};
+/************************************************************************/
 
 struct	sstat {
 	struct cpustat	cpu;
@@ -287,6 +356,9 @@ struct	sstat {
 	struct dskstat  dsk;
 	struct nfsstat  nfs;
 	struct contstat cfs;
+	struct pressure	psi;
+	struct gpustat 	gpu;
+	struct ifbstat 	ifb;
 
 	struct wwwstat	www;
 };
@@ -297,3 +369,4 @@ struct	sstat {
 void	photosyst (struct sstat *);
 void	deviatsyst(struct sstat *, struct sstat *, struct sstat *, long);
 void	totalsyst (char,           struct sstat *, struct sstat *);
+#endif
