@@ -14,7 +14,6 @@
 **
 ** Author:	JC van Winkel - AT Computing, Nijmegen, Holland
 **              Complete redesign.
-** E-mail:      jc@ATComputing.nl
 ** Date:        November 2009
 ** --------------------------------------------------------------------------
 ** Copyright (C) 2009-2010 JC van Winkel
@@ -294,6 +293,7 @@ static void	make_proc_dynamicgen(void);
 ** these defaults can be overruled via the config-file
 */
 int   cpubadness = 90;        /* percentage           */
+int   gpubadness = 100;       /* percentage           */
 int   membadness = 90;        /* percentage           */
 int   swpbadness = 80;        /* percentage           */
 int   dskbadness = 70;        /* percentage           */
@@ -325,6 +325,8 @@ sys_printdef *cpusyspdefs[] = {
 	&syspdef_CPUIDLE,
 	&syspdef_CPUWAIT,
 	&syspdef_BLANKBOX,
+	&syspdef_CPUIPC,
+	&syspdef_CPUCYCLE,
 	&syspdef_CPUFREQ,
 	&syspdef_CPUSCALE,
 	&syspdef_CPUSTEAL,
@@ -339,6 +341,8 @@ sys_printdef *cpisyspdefs[] = {
 	&syspdef_CPUIIDLE,
 	&syspdef_CPUIWAIT,
 	&syspdef_BLANKBOX,
+	&syspdef_CPUIIPC,
+	&syspdef_CPUICYCLE,
 	&syspdef_CPUIFREQ,
 	&syspdef_CPUISCALE,
 	&syspdef_CPUISTEAL,
@@ -356,6 +360,19 @@ sys_printdef *cplsyspdefs[] = {
 	&syspdef_BLANKBOX,
         0
 };
+sys_printdef *gpusyspdefs[] = {
+	&syspdef_GPUBUS,
+	&syspdef_GPUGPUPERC,
+	&syspdef_GPUMEMPERC,
+	&syspdef_GPUMEMOCC,
+	&syspdef_GPUMEMTOT,
+	&syspdef_GPUMEMUSE,
+	&syspdef_GPUMEMAVG,
+	&syspdef_GPUTYPE,
+	&syspdef_GPUNRPROC,
+	&syspdef_BLANKBOX,
+        0
+};
 sys_printdef *memsyspdefs[] = {
 	&syspdef_MEMTOT,
 	&syspdef_MEMFREE,
@@ -392,6 +409,15 @@ sys_printdef *pagsyspdefs[] = {
 	&syspdef_BLANKBOX,
         0
 };
+sys_printdef *psisyspdefs[] = {
+	&syspdef_PSICPUS,
+	&syspdef_PSIMEMS,
+	&syspdef_PSIMEMF,
+	&syspdef_PSIIOS,
+	&syspdef_PSIIOF,
+	&syspdef_BLANKBOX,
+        0
+};
 sys_printdef *contsyspdefs[] = {
 	&syspdef_CONTNAME,
 	&syspdef_CONTNPROC,
@@ -498,6 +524,17 @@ sys_printdef *netintfsyspdefs[] = {
         0
 };
 
+sys_printdef *infinisyspdefs[] = {
+	&syspdef_IFBNAME,
+	&syspdef_IFBPCKI,
+	&syspdef_IFBPCKO,
+	&syspdef_IFBSPEEDMAX,
+	&syspdef_IFBSPEEDIN,
+	&syspdef_IFBSPEEDOUT,
+	&syspdef_IFBLANES,
+	&syspdef_BLANKBOX,
+        0
+};
 
 /*
  * table with all proc_printdefs
@@ -567,6 +604,11 @@ proc_printdef *allprocpdefs[]=
 	&procprt_SNET,
 	&procprt_BANDWI,
 	&procprt_BANDWO,
+	&procprt_GPULIST,
+	&procprt_GPUMEMNOW,
+	&procprt_GPUMEMAVG,
+	&procprt_GPUGPUBUSY,
+	&procprt_GPUMEMBUSY,
 	&procprt_SORTITEM,
         0
 };
@@ -595,6 +637,7 @@ proc_printpair schedprocs[MAXITEMS];
 proc_printpair genprocs[MAXITEMS];
 proc_printpair dskprocs[MAXITEMS];
 proc_printpair netprocs[MAXITEMS];
+proc_printpair gpuprocs[MAXITEMS];
 proc_printpair varprocs[MAXITEMS];
 proc_printpair cmdprocs[MAXITEMS];
 proc_printpair ownprocs[MAXITEMS];
@@ -612,14 +655,17 @@ sys_printpair sysprcline[MAXITEMS];
 sys_printpair allcpuline[MAXITEMS];
 sys_printpair indivcpuline[MAXITEMS];
 sys_printpair cplline[MAXITEMS];
+sys_printpair gpuline[MAXITEMS];
 sys_printpair memline[MAXITEMS];
 sys_printpair swpline[MAXITEMS];
 sys_printpair pagline[MAXITEMS];
+sys_printpair psiline[MAXITEMS];
 sys_printpair contline[MAXITEMS];
 sys_printpair dskline[MAXITEMS];
 sys_printpair nettransportline[MAXITEMS];
 sys_printpair netnetline[MAXITEMS];
 sys_printpair netinterfaceline[MAXITEMS];
+sys_printpair infinibandline[MAXITEMS];
 sys_printpair nfsmountline[MAXITEMS];
 sys_printpair nfcline[MAXITEMS];
 sys_printpair nfsline[MAXITEMS];
@@ -779,7 +825,9 @@ init_proc_prints()
 
 		if ( strlen(idprocpdefs[i]->head) < numdigits)
 		{
-			char *p = malloc(numdigits) + 1;
+			char *p = malloc(numdigits+1);
+
+			ptrverify(p, "Malloc failed for formatted header\n");
 
 			sprintf(p, "%*s", numdigits, idprocpdefs[i]->head);
 			idprocpdefs[i]->head = p;
@@ -840,6 +888,7 @@ totalcap(struct syscap *psc, struct sstat *sstat,
         register int    i;
 
         psc->nrcpu      = sstat->cpu.nrcpu;
+
         psc->availcpu   = sstat->cpu.all.stime +
                           sstat->cpu.all.utime +
                           sstat->cpu.all.ntime +
@@ -874,6 +923,11 @@ totalcap(struct syscap *psc, struct sstat *sstat,
 		psc->availdsk += curstat->dsk.rsz;
 		psc->availdsk += nett_wsz;
 	}
+
+	for (psc->availgpumem=i=0; i < sstat->gpu.nrgpus; i++)
+		psc->availgpumem += sstat->gpu.gpu[i].memtotnow;
+
+	psc->nrgpu = sstat->gpu.nrgpus;
 }
 
 /*
@@ -910,13 +964,15 @@ pricumproc(struct sstat *sstat, struct devtstat *devtstat,
                     make_sys_prints(allcpuline, MAXITEMS,
 	                "CPUSYS:9 "
 	                "CPUUSER:8 "
-	                "CPUIRQ:5 "
-	                "BLANKBOX:0 "
-	                "CPUIDLE:6 "
-	                "CPUWAIT:6 "
+	                "CPUIRQ:6 "
 	                "BLANKBOX:0 "
+	                "CPUIDLE:7 "
+	                "CPUWAIT:7 "
                         "CPUSTEAL:2 "
                         "CPUGUEST:3 "
+	                "BLANKBOX:0 "
+                        "CPUIPC:5 "
+                        "CPUCYCLE:4 "
                         "CPUFREQ:4 "
                         "CPUSCALE:4 ", cpusyspdefs, "builtin allcpuline");
                 }
@@ -926,13 +982,15 @@ pricumproc(struct sstat *sstat, struct devtstat *devtstat,
                     make_sys_prints(indivcpuline, MAXITEMS,
 	                "CPUISYS:9 "
                         "CPUIUSER:8 "
-	                "CPUIIRQ:5 "
-	                "BLANKBOX:0 "
-	                "CPUIIDLE:6 "
-	                "CPUIWAIT:6 "
+	                "CPUIIRQ:6 "
 	                "BLANKBOX:0 "
+	                "CPUIIDLE:7 "
+	                "CPUIWAIT:7 "
                         "CPUISTEAL:2 "
                         "CPUIGUEST:3 "
+	                "BLANKBOX:0 "
+                        "CPUIIPC:5 "
+                        "CPUICYCLE:4 "
                         "CPUIFREQ:4 "
                         "CPUISCALE:4 ", cpisyspdefs, "builtin indivcpuline");
                 }
@@ -950,6 +1008,21 @@ pricumproc(struct sstat *sstat, struct devtstat *devtstat,
 	                "CPLNUMCPU:1", cplsyspdefs, "builtin cplline");
                 }
 
+                if (gpuline[0].f == 0)
+                {
+                    make_sys_prints(gpuline, MAXITEMS,
+	                "GPUBUS:8 "
+	                "GPUGPUPERC:7 "
+	                "GPUMEMPERC:6 "
+	                "GPUMEMOCC:5 "
+	                "GPUMEMTOT:3 "
+	                "GPUMEMUSE:4 "
+	                "GPUMEMAVG:2 "
+	                "GPUNRPROC:2 "
+	                "BLANKBOX:0 "
+	                "GPUTYPE:1 ", gpusyspdefs, "builtin gpuline");
+                }
+
                 if (memline[0].f == 0)
                 {
                     make_sys_prints(memline, MAXITEMS,
@@ -998,6 +1071,18 @@ pricumproc(struct sstat *sstat, struct devtstat *devtstat,
 	                "PAGSWIN:3 "
 	                "PAGSWOUT:4", pagsyspdefs, "builtin pagline");
                 }
+                if (psiline[0].f == 0)
+                {
+                    make_sys_prints(psiline, MAXITEMS,
+	                "PSICPUS:3 "
+	                "PSIMEMS:3 "
+	                "PSIMEMF:3 "
+	                "PSIIOS:3 "
+	                "PSIIOF:3 "
+	                "BLANKBOX:0 "
+	                "BLANKBOX:0 "
+	                "BLANKBOX:0 ", psisyspdefs, "builtin psiline");
+                }
                 if (contline[0].f == 0)
                 {
                     make_sys_prints(contline, MAXITEMS,
@@ -1125,6 +1210,25 @@ pricumproc(struct sstat *sstat, struct devtstat *devtstat,
                         "NETRCVDROP:3 "
                         "NETSNDDROP:3", netintfsyspdefs, "builtin netinterfaceline");
                 }
+                if (infinibandline[0].f == 0)
+                {
+                    make_sys_prints(infinibandline, MAXITEMS,
+	                "IFBNAME:8 "
+	                "BLANKBOX:0 "
+	                "IFBPCKI:7 "
+	                "IFBPCKO:7 "
+	                "BLANKBOX:0 "
+	                "IFBSPEEDMAX:5 "
+	                "IFBSPEEDIN:6 "
+	                "IFBSPEEDOUT:6 "
+	                "IFBLANES:4 "
+	                "BLANKBOX:0 "
+	                "BLANKBOX:0 "
+	                "BLANKBOX:0 "
+	                "BLANKBOX:0 "
+	                "BLANKBOX:0 "
+	                "BLANKBOX:0 ", infinisyspdefs, "builtin infinibandline");
+                }
         }  // firsttime
 
 
@@ -1199,6 +1303,11 @@ priphead(int curlist, int totlist, char *showtype, char *showorder,
                         "SORTITEM:10 CMD:10", 
                         "built-in netprocs");
 
+                make_proc_prints(gpuprocs, MAXITEMS, 
+                        "PID:10 TID:5 CID:4 GPULIST:8 GPUGPUBUSY:8 GPUMEMBUSY:8 "
+			"GPUMEM:7 GPUMEMAVG:6 S:8 SORTITEM:10 CMD:10", 
+                        "built-in gpuprocs");
+
                 make_proc_prints(varprocs, MAXITEMS,
                         "PID:10 TID:4 PPID:9 CID:2 VPID:1 CTID:1 "
 			"RUID:8 RGID:8 EUID:5 EGID:4 "
@@ -1268,6 +1377,10 @@ priphead(int curlist, int totlist, char *showtype, char *showorder,
                 showhdrline(netprocs, curlist, totlist, *showorder, autosort);
                 break;
 
+           case MPROCGPU:
+                showhdrline(gpuprocs, curlist, totlist, *showorder, autosort);
+                break;
+
            case MPROCVAR:
                 showhdrline(varprocs, curlist, totlist, *showorder, autosort);
                 break;
@@ -1456,6 +1569,23 @@ priproc(struct tstat **proclist, int firstproc, int lastproc, int curline,
                         }
                         break;
 
+                   case MSORTGPU:
+                        perc = 0.0;
+
+			if (!curstat->gpu.state)
+				break;
+
+                        if (curstat->gpu.gpubusy != -1)
+			{
+                        	perc = curstat->gpu.gpubusy;
+			}
+			else
+			{
+                        	perc = curstat->gpu.memnow*100 *
+				       sb->nrgpu / sb->availgpumem;
+			}
+                        break;
+
                    default:
                         perc = 0.0;
                 }
@@ -1485,6 +1615,10 @@ priproc(struct tstat **proclist, int firstproc, int lastproc, int curline,
                         showprocline(netprocs, curstat, perc, nsecs, avgval);
                         break;
 
+                   case MPROCGPU:
+                        showprocline(gpuprocs, curstat, perc, nsecs, avgval);
+                        break;
+
                    case MPROCVAR:
                         showprocline(varprocs, curstat, perc, nsecs, avgval);
                         break;
@@ -1530,8 +1664,9 @@ static void	pridisklike(extraparam *, struct perdsk *, char *,
 int
 prisyst(struct sstat *sstat, int curline, int nsecs, int avgval,
         int fixedhead, struct sselection *selp, char *highorderp,
-        int maxcpulines, int maxdsklines, int maxmddlines,
-	int maxlvmlines, int maxintlines, int maxnfslines, int maxcontlines)
+        int maxcpulines, int maxgpulines, int maxdsklines, int maxmddlines,
+	int maxlvmlines, int maxintlines, int maxifblines,
+	int maxnfslines, int maxcontlines)
 {
         extraparam      extra;
         int             lin;
@@ -1618,7 +1753,9 @@ prisyst(struct sstat *sstat, int curline, int nsecs, int avgval,
                                 extra.percputot = 1; /* avoid divide-by-zero */
 
 
-                        move(curline, 0);
+			if (screen)
+        	                move(curline, 0);
+
                         showsysline(indivcpuline, sstat, &extra, "cpu",
 								badness);
                         curline++;
@@ -1635,6 +1772,65 @@ prisyst(struct sstat *sstat, int curline, int nsecs, int avgval,
         showsysline(cplline, sstat, &extra, "CPL", 0);
         curline++;
 
+        /*
+        ** GPU statistics
+        */
+	if (sstat->gpu.nrgpus)
+	{
+        	for (extra.index=0, lin=0;
+		     extra.index < sstat->gpu.nrgpus && lin < maxgpulines;
+		     extra.index++)
+        	{
+			int	totbusy;
+			count_t	avgmemuse;
+
+			// notice that GPU percentage and memory percentage
+			// are not always available; in that case both
+			// values have the value -1
+			//
+			totbusy = sstat->gpu.gpu[extra.index].gpuperccum +
+			          sstat->gpu.gpu[extra.index].memperccum;
+
+			if (totbusy == -2)	// metrics available?
+				totbusy= 0;
+
+			if (sstat->gpu.gpu[extra.index].samples == 0)
+			{
+				totbusy =
+					sstat->gpu.gpu[extra.index].gpupercnow +
+			          	sstat->gpu.gpu[extra.index].mempercnow;
+
+				avgmemuse =
+					sstat->gpu.gpu[extra.index].memusenow;
+			}
+			else
+			{
+				totbusy = totbusy /
+				     	sstat->gpu.gpu[extra.index].samples;
+
+		   		avgmemuse =
+					 sstat->gpu.gpu[extra.index].memusecum/
+			                 sstat->gpu.gpu[extra.index].samples;
+			}
+
+        		if (gpubadness)
+                		badness = totbusy * 100 / gpubadness;
+        		else
+                		badness = 0;
+
+			if (	totbusy > 0 			||
+			    	// memusage > 512 MiB (rather arbitrary)?
+			    	avgmemuse > 512*1024 		||
+			    	fixedhead			  )
+			{
+					showsysline(gpuline, sstat,
+							&extra, "GPU", badness);
+	 				curline++;
+					lin++;
+			}
+		}
+	}
+
         /*
         ** MEMORY statistics
         */
@@ -1730,6 +1926,35 @@ prisyst(struct sstat *sstat, int curline, int nsecs, int avgval,
                 curline++;
         }
 
+        /*
+        ** Pressure statistics
+        */
+	if (sstat->psi.present)
+	{
+        	if (fixedhead                 ||
+	            sstat->psi.cpusome.avg10  || sstat->psi.memsome.avg10  ||
+	            sstat->psi.iosome.avg10   ||
+	            sstat->psi.cpusome.avg60  || sstat->psi.memsome.avg60  ||
+	            sstat->psi.iosome.avg60   ||
+	            sstat->psi.cpusome.avg300 || sstat->psi.memsome.avg300 ||
+	            sstat->psi.iosome.avg300    )
+	        {
+			badness = sstat->psi.cpusome.avg10 >
+			          sstat->psi.cpusome.avg60 ?
+			          sstat->psi.cpusome.avg10 :
+			          sstat->psi.cpusome.avg60;
+
+			if (badness < sstat->psi.cpusome.avg300)
+				badness = sstat->psi.cpusome.avg300;
+
+			if (screen)
+                		move(curline, 0);
+
+                	showsysline(psiline, sstat, &extra,"PSI", badness);
+                	curline++;
+		}
+	}
+
 	/*
  	** Container statistics (if any)
 	*/
@@ -1815,7 +2040,7 @@ prisyst(struct sstat *sstat, int curline, int nsecs, int avgval,
         }
 
         /*
-        ** NET statistics
+        ** NET statistics: transport
         */
         if (sstat->net.tcp.InSegs             ||
             sstat->net.tcp.OutSegs            ||
@@ -1832,6 +2057,9 @@ prisyst(struct sstat *sstat, int curline, int nsecs, int avgval,
                 curline++;
         }
 
+        /*
+        ** NET statistics: network
+        */
         if (sstat->net.ipv4.InReceives ||
             sstat->net.ipv6.Ip6InReceives ||
             sstat->net.ipv4.OutRequests ||
@@ -1845,6 +2073,9 @@ prisyst(struct sstat *sstat, int curline, int nsecs, int avgval,
                 curline++;
         }
 
+        /*
+        ** NET statistics: interfaces
+        */
         for (extra.index=0, lin=0;
 	     sstat->intf.intf[extra.index].name[0] && lin < maxintlines;
              extra.index++)
@@ -1909,6 +2140,54 @@ prisyst(struct sstat *sstat, int curline, int nsecs, int avgval,
                 }
         }
 
+        /*
+        ** NET statistics: InfiniBand
+        */
+        for (extra.index=0, lin=0;
+	     extra.index < sstat->ifb.nrports && lin < maxifblines;
+             extra.index++)
+        {
+                if (sstat->ifb.ifb[extra.index].rcvb ||
+                    sstat->ifb.ifb[extra.index].sndb || fixedhead)
+                {
+                        /*
+                        ** calculate busy-percentage for IB port
+                        */
+                       count_t ival, oval;
+
+                        /*
+                        ** convert byte-transfers to bit-transfers     (*    8)
+                        ** convert bit-transfers  to kilobit-transfers (/ 1000)
+                        ** per second
+                        */
+                        ival    = sstat->ifb.ifb[extra.index].rcvb/125/nsecs;
+                        oval    = sstat->ifb.ifb[extra.index].sndb/125/nsecs;
+
+			busy = (ival > oval ? ival : oval) *
+			                 sstat->ifb.ifb[extra.index].lanes /
+					(sstat->ifb.ifb[extra.index].rate * 10);
+
+                        if (netbadness)
+                                badness = busy * 100 / netbadness;
+                        else
+                                badness = 0;
+
+                        if (highbadness < badness)
+                        {
+                                highbadness = badness;
+                                *highorderp = MSORTNET;
+                        }
+
+			if (screen)
+                		move(curline, 0);
+
+                        showsysline(infinibandline, sstat, &extra, 
+                                      			"IFB", badness);
+                        curline++;
+                        lin++;
+                }
+        }
+
         /*
         ** application statistics
         **
@@ -1999,7 +2278,7 @@ pridisklike(extraparam *ep, struct perdsk *dp, char *lp, char *highorderp,
 
 
 /*
-** sort-functions
+** process-level sort functions
 */
 int
 compcpu(const void *a, const void *b)
@@ -2049,6 +2328,36 @@ compmem(const void *a, const void *b)
                          return  0;
 }
 
+int
+compgpu(const void *a, const void *b)
+{
+        register char 	 astate = (*(struct tstat **)a)->gpu.state;
+        register char 	 bstate = (*(struct tstat **)b)->gpu.state;
+        register count_t abusy  = (*(struct tstat **)a)->gpu.gpubusy;
+        register count_t bbusy  = (*(struct tstat **)b)->gpu.gpubusy;
+        register count_t amem   = (*(struct tstat **)a)->gpu.memnow;
+        register count_t bmem   = (*(struct tstat **)b)->gpu.memnow;
+
+        if (!astate)		// no GPU usage?
+		abusy = amem = -2; 
+
+        if (!bstate)		// no GPU usage?
+		bbusy = bmem = -2; 
+
+	if (abusy == -1 || bbusy == -1)
+	{
+        	if (amem < bmem)	return  1;
+	        if (amem > bmem) 	return -1;
+                         		return  0;
+	}
+	else
+	{
+		if (abusy < bbusy)	return  1;
+		if (abusy > bbusy)	return -1;
+       		                  	return  0;
+	}
+}
+
 int
 compnet(const void *a, const void *b)
 {
@@ -2095,6 +2404,9 @@ compcon(const void *a, const void *b)
        return strcmp(containera, containerb);
 }
 
+/*
+** system-level sort functions
+*/
 int
 cpucompar(const void *a, const void *b)
 {
@@ -2108,6 +2420,28 @@ cpucompar(const void *a, const void *b)
                            return  0;
 }
 
+int
+gpucompar(const void *a, const void *b)
+{
+        register count_t agpuperc  = ((struct pergpu *)a)->gpuperccum;
+        register count_t bgpuperc  = ((struct pergpu *)b)->gpuperccum;
+        register count_t amemuse   = ((struct pergpu *)a)->memusenow;
+        register count_t bmemuse   = ((struct pergpu *)b)->memusenow;
+
+	if (agpuperc == -1 || bgpuperc == -1)
+	{
+        	if (amemuse < bmemuse)	return  1;
+        	if (amemuse > bmemuse)	return -1;
+                      			return  0;
+	}
+	else
+	{
+        	if (agpuperc < bgpuperc)	return  1;
+        	if (agpuperc > bgpuperc)	return -1;
+                      				return  0;
+	}
+}
+
 int
 diskcompar(const void *a, const void *b)
 {
@@ -2133,7 +2467,6 @@ intfcompar(const void *a, const void *b)
         count_t asbyte         = ((struct perintf *)a)->sbyte;
         count_t bsbyte         = ((struct perintf *)b)->sbyte;
 
-
         /*
         ** if speed of first interface known, calculate busy factor
         */
@@ -2181,6 +2514,21 @@ intfcompar(const void *a, const void *b)
                 return  1;
 }
 
+
+int
+ifbcompar(const void *a, const void *b)
+{
+        count_t atransfer  = ((struct perifb *)a)->rcvb +
+                             ((struct perifb *)a)->sndb;
+        count_t btransfer  = ((struct perifb *)b)->rcvb +
+                             ((struct perifb *)b)->sndb;
+
+	if (atransfer < btransfer)	return  1;
+	if (atransfer > btransfer)	return -1;
+					return  0;
+}
+
+
 int
 nfsmcompar(const void *a, const void *b)
 {
@@ -2262,6 +2610,12 @@ do_cpucritperc(char *name, char *val)
         cpubadness = get_perc(name, val);
 }
 
+void
+do_gpucritperc(char *name, char *val)
+{
+        gpubadness = get_perc(name, val);
+}
+
 void
 do_memcritperc(char *name, char *val)
 {
@@ -2322,6 +2676,12 @@ do_owncplline(char *name, char *val)
         make_sys_prints(cplline, MAXITEMS, val, cplsyspdefs, name);
 }
 
+void
+do_owngpuline(char *name, char *val)
+{
+        make_sys_prints(gpuline, MAXITEMS, val, gpusyspdefs, name);
+}
+
 void
 do_ownmemline(char *name, char *val)
 {
@@ -2364,6 +2724,12 @@ do_ownnetinterfaceline(char *name, char *val)
         make_sys_prints(netinterfaceline, MAXITEMS, val, netintfsyspdefs, name);
 }
 
+void
+do_owninfinibandline(char *name, char *val)
+{
+        make_sys_prints(infinibandline, MAXITEMS, val, infinisyspdefs, name);
+}
+
 void
 do_ownprocline(char *name, char *val)
 {
