@@ -129,6 +129,7 @@ extern sys_printdef *dsksyspdefs[];
 extern sys_printdef *nettranssyspdefs[];
 extern sys_printdef *netnetsyspdefs[];
 extern sys_printdef *netintfsyspdefs[];
+extern sys_printdef *infinisyspdefs[];
 
 extern sys_printdef syspdef_PRCSYS;
 extern sys_printdef syspdef_PRCUSER;
@@ -158,12 +159,25 @@ extern sys_printdef syspdef_CPUSTEAL;
 extern sys_printdef syspdef_CPUISTEAL;
 extern sys_printdef syspdef_CPUGUEST;
 extern sys_printdef syspdef_CPUIGUEST;
+extern sys_printdef syspdef_CPUIPC;
+extern sys_printdef syspdef_CPUIIPC;
+extern sys_printdef syspdef_CPUCYCLE;
+extern sys_printdef syspdef_CPUICYCLE;
 extern sys_printdef syspdef_CPLAVG1;
 extern sys_printdef syspdef_CPLAVG5;
 extern sys_printdef syspdef_CPLAVG15;
 extern sys_printdef syspdef_CPLCSW;
 extern sys_printdef syspdef_CPLNUMCPU;
 extern sys_printdef syspdef_CPLINTR;
+extern sys_printdef syspdef_GPUBUS;
+extern sys_printdef syspdef_GPUTYPE;
+extern sys_printdef syspdef_GPUNRPROC;
+extern sys_printdef syspdef_GPUMEMPERC;
+extern sys_printdef syspdef_GPUMEMOCC;
+extern sys_printdef syspdef_GPUGPUPERC;
+extern sys_printdef syspdef_GPUMEMTOT;
+extern sys_printdef syspdef_GPUMEMUSE;
+extern sys_printdef syspdef_GPUMEMAVG;
 extern sys_printdef syspdef_MEMTOT;
 extern sys_printdef syspdef_MEMFREE;
 extern sys_printdef syspdef_MEMCACHE;
@@ -186,6 +200,11 @@ extern sys_printdef syspdef_PAGSTEAL;
 extern sys_printdef syspdef_PAGSTALL;
 extern sys_printdef syspdef_PAGSWIN;
 extern sys_printdef syspdef_PAGSWOUT;
+extern sys_printdef syspdef_PSICPUS;
+extern sys_printdef syspdef_PSIMEMS;
+extern sys_printdef syspdef_PSIMEMF;
+extern sys_printdef syspdef_PSIIOS;
+extern sys_printdef syspdef_PSIIOF;
 extern sys_printdef syspdef_CONTNAME;
 extern sys_printdef syspdef_CONTNPROC;
 extern sys_printdef syspdef_CONTCPU;
@@ -259,6 +278,13 @@ extern sys_printdef syspdef_NFSNWBYTES;
 extern sys_printdef syspdef_NFSRCHITS;
 extern sys_printdef syspdef_NFSRCMISS;
 extern sys_printdef syspdef_NFSRCNOCA;
+extern sys_printdef syspdef_IFBNAME;
+extern sys_printdef syspdef_IFBPCKI;
+extern sys_printdef syspdef_IFBPCKO;
+extern sys_printdef syspdef_IFBSPEEDMAX;
+extern sys_printdef syspdef_IFBLANES;
+extern sys_printdef syspdef_IFBSPEEDIN;
+extern sys_printdef syspdef_IFBSPEEDOUT;
 extern sys_printdef syspdef_BLANKBOX;
 
 
@@ -334,6 +360,11 @@ extern proc_printdef procprt_RNET;
 extern proc_printdef procprt_SNET;
 extern proc_printdef procprt_BANDWI;
 extern proc_printdef procprt_BANDWO;
+extern proc_printdef procprt_GPULIST;
+extern proc_printdef procprt_GPUMEMNOW;
+extern proc_printdef procprt_GPUMEMAVG;
+extern proc_printdef procprt_GPUGPUBUSY;
+extern proc_printdef procprt_GPUMEMBUSY;
 extern proc_printdef procprt_SORTITEM;
 
 
