@@ -112,6 +112,7 @@ static void	format_bandw(char *, count_t);
 static char     *columnhead[] = {
 	[MSORTCPU]= "CPU", [MSORTMEM]= "MEM",
 	[MSORTDSK]= "DSK", [MSORTNET]= "NET",
+	[MSORTGPU]= "GPU",
 };
 
 
@@ -1204,6 +1205,7 @@ proc_printdef procprt_TSLPU =
 #define SCHED_BATCH	3
 #define SCHED_ISO	4
 #define SCHED_IDLE	5
+#define SCHED_DEADLINE	6
 
 char *
 procprt_POLI_a(struct tstat *curstat, int avgval, int nsecs)
@@ -1228,6 +1230,9 @@ procprt_POLI_a(struct tstat *curstat, int avgval, int nsecs)
                 case SCHED_IDLE:
                         return "idle";
                         break;
+                case SCHED_DEADLINE:
+                        return "dead";
+                        break;
         }
         return "?   ";
 }
@@ -1634,7 +1639,6 @@ procprt_TCPSASZ_e(struct tstat *curstat, int avgval, int nsecs)
         	return "      -";
 }
 
-
 proc_printdef procprt_TCPSASZ = 
    { "TCPSASZ", "TCPSASZ", procprt_TCPSASZ_a, procprt_TCPSASZ_e, 7 };
 /***************************************************************/
@@ -1906,6 +1910,114 @@ format_bandw(char *buf, count_t kbps)
 }
 /***************************************************************/
 char *
+procprt_GPULIST_ae(struct tstat *curstat, int avgval, int nsecs)
+{
+        static char	buf[64];
+        char		tmp[64], *p=tmp;
+	int		i;
+
+	if (!curstat->gpu.state)
+		return "       -";
+
+	if (!curstat->gpu.gpulist)
+		return "       -";
+
+	for (i=0; i < nrgpus; i++)
+	{
+		if (curstat->gpu.gpulist & 1<<i)
+		{
+			if (tmp == p)	// first?
+				p += snprintf(p, sizeof tmp, "%d", i);
+			else
+				p += snprintf(p, sizeof tmp - (p-tmp), ",%d", i);
+
+			if (p - tmp > 8)
+			{
+				snprintf(tmp, sizeof tmp, "0x%06x",
+						curstat->gpu.gpulist);
+				break;
+			}
+		}
+	}
+
+	snprintf(buf, sizeof buf, "%8.8s", tmp);
+        return buf;
+}
+
+proc_printdef procprt_GPULIST = 
+   { " GPUNUMS", "GPULIST", procprt_GPULIST_ae, procprt_GPULIST_ae, 8};
+/***************************************************************/
+char *
+procprt_GPUMEMNOW_ae(struct tstat *curstat, int avgval, int nsecs)
+{
+        static char buf[10];
+
+	if (!curstat->gpu.state)
+		return "     -";
+
+        val2memstr(curstat->gpu.memnow*1024, buf, KBFORMAT, 0, 0);
+        return buf;
+}
+
+proc_printdef procprt_GPUMEMNOW = 
+   { "MEMNOW", "GPUMEM", procprt_GPUMEMNOW_ae, procprt_GPUMEMNOW_ae, 6};
+/***************************************************************/
+char *
+procprt_GPUMEMAVG_ae(struct tstat *curstat, int avgval, int nsecs)
+{
+        static char buf[10];
+
+	if (!curstat->gpu.state)
+		return "     -";
+
+	if (curstat->gpu.sample == 0)
+		return("    0K");
+
+       	val2memstr(curstat->gpu.nrgpus * curstat->gpu.memcum /
+	           curstat->gpu.sample*1024, buf, KBFORMAT, 0, 0);
+       	return buf;
+}
+
+proc_printdef procprt_GPUMEMAVG = 
+   { "MEMAVG", "GPUMEMAVG", procprt_GPUMEMAVG_ae, procprt_GPUMEMAVG_ae, 6};
+/***************************************************************/
+char *
+procprt_GPUGPUBUSY_ae(struct tstat *curstat, int avgval, int nsecs)
+{
+        static char 	buf[16];
+
+	if (!curstat->gpu.state)
+		return "      -";
+
+	if (curstat->gpu.gpubusy == -1)
+		return "    N/A";
+
+       	snprintf(buf, sizeof buf, "%6d%%", curstat->gpu.gpubusy);
+       	return buf;
+}
+
+proc_printdef procprt_GPUGPUBUSY = 
+   { "GPUBUSY", "GPUGPUBUSY", procprt_GPUGPUBUSY_ae, procprt_GPUGPUBUSY_ae, 7};
+/***************************************************************/
+char *
+procprt_GPUMEMBUSY_ae(struct tstat *curstat, int avgval, int nsecs)
+{
+        static char 	buf[16];
+
+	if (!curstat->gpu.state)
+		return "      -";
+
+	if (curstat->gpu.membusy == -1)
+		return "    N/A";
+
+        snprintf(buf, sizeof buf, "%6d%%", curstat->gpu.membusy);
+        return buf;
+}
+
+proc_printdef procprt_GPUMEMBUSY = 
+   { "MEMBUSY", "GPUMEMBUSY", procprt_GPUMEMBUSY_ae, procprt_GPUMEMBUSY_ae, 7};
+/***************************************************************/
+char *
 procprt_SORTITEM_ae(struct tstat *curstat, int avgval, int nsecs)
 {
         return "";   // dummy function
