@@ -98,8 +98,6 @@
 **
 */
 
-static const char rcsid[] = "$Id: various.c,v 1.21 2010/11/12 06:16:16 gerlof Exp $";
-
 #include <sys/types.h>
 #include <sys/param.h>
 #include <sys/stat.h>
@@ -366,11 +364,13 @@ val2cpustr(count_t value, char *strvalue)
 ** Function val2Hzstr() converts a value (in MHz) 
 ** to an ascii-string.
 ** The result-string is placed in the area pointed to strvalue,
-** which should be able to contain at least 8 positions.
+** which should be able to contain 7 positions plus null byte.
 */
 char *
 val2Hzstr(count_t value, char *strvalue)
 {
+	char *fformat;
+
         if (value < 1000)
         {
                 sprintf(strvalue, "%4lldMHz", value);
@@ -385,8 +385,22 @@ val2Hzstr(count_t value, char *strvalue)
                         prefix='T';        
                         fval /= 1000.0;
                 }
-                sprintf(strvalue, "%4.2f%cHz", fval, prefix);
+
+                if (fval < 10.0)
+		{
+			 fformat = "%4.2f%cHz";
+		}
+		else
+		{
+			if (fval < 100.0)
+				fformat = "%4.1f%cHz";
+                	else
+				fformat = "%4.0f%cHz";
+		}
+
+                sprintf(strvalue, fformat, fval, prefix);
         }
+
 	return strvalue;
 }
 
@@ -546,21 +560,19 @@ getboot(void)
 void
 ptrverify(const void *ptr, const char *errormsg, ...)
 {
-	va_list args;
-
-        va_start(args, errormsg);
-
 	if (!ptr)
 	{
+		va_list args;
+
 		acctswoff();
 		netatop_signoff();
 
 		if (vis.show_end)
 			(vis.show_end)();
 
-        	va_list args;
-		fprintf(stderr, errormsg, args);
-        	va_end  (args);
+		va_start(args, errormsg);
+		vfprintf(stderr, errormsg, args);
+		va_end  (args);
 
 		exit(13);
 	}
